﻿Imports System.Runtime.InteropServices
Imports System.Security.Permissions
Imports System.Text

<SecurityPermission(SecurityAction.InheritanceDemand, Flags:=SecurityPermissionFlag.UnmanagedCode)> _
Public Class WindowsCredentialPrompt
    Implements IDisposable

    Private m_Caption As String = String.Empty
    Private m_Message As String = String.Empty
    Private m_Parent As IntPtr = IntPtr.Zero

    Private m_UserName As String = String.Empty
    Private m_Password As String = String.Empty
    Private m_Domain As String = String.Empty

    Public Enum PromptResult
        OK = 0
        Cancel = 1223
    End Enum

    Public Sub New(ByVal caption As String, ByVal message As String, ByVal parent As IntPtr)
        Me.m_Caption = caption
        Me.m_Message = message
        Me.m_Parent = parent
    End Sub

    <SecurityPermission(SecurityAction.LinkDemand, Flags:=SecurityPermissionFlag.UnmanagedCode)> _
    Public Function Show() As PromptResult

        Dim ci As SafeNativeMethods.CredUIInfo = Nothing
        Dim result As Integer = 0
        Dim ppvOutAuthBuffer As IntPtr = IntPtr.Zero
        Dim pulOutAuthBufferSize As Integer = 0
        Dim pulAuthPackage As Integer = 0
        Dim pfSave As Boolean = False

        ci.cbSize = Marshal.SizeOf(ci)
        ci.hbmBanner = IntPtr.Zero
        ci.hwndParent = m_Parent
        ci.pszCaptionText = m_Caption
        ci.pszMessageText = m_Message

        result = SafeNativeMethods.CredUIPromptForWindowsCredentials(ci, 0, pulAuthPackage, IntPtr.Zero, 0, ppvOutAuthBuffer, pulOutAuthBufferSize, pfSave, SafeNativeMethods.CredUIFlags.Generic)

        If ppvOutAuthBuffer <> IntPtr.Zero Then

            Dim username As New StringBuilder(SafeNativeMethods.CredMaxUsernameLength)
            Dim password As New StringBuilder(SafeNativeMethods.CredMaxPasswordLength)
            Dim domain As New StringBuilder(SafeNativeMethods.CredMaxDomainNameLength)

            SafeNativeMethods.CredUnPackAuthenticationBuffer(0, ppvOutAuthBuffer, pulOutAuthBufferSize, username, username.Capacity, domain, domain.Capacity, password, password.Capacity)
            m_UserName = username.ToString
            m_Password = password.ToString
            m_Domain = domain.ToString
            Marshal.FreeCoTaskMem(ppvOutAuthBuffer)
        End If

        Return CType(result, PromptResult)
    End Function


    ReadOnly Property UserName() As String
        Get
            Return m_UserName
        End Get
    End Property

    ReadOnly Property Password() As String
        Get
            Return m_Password
        End Get
    End Property

    ReadOnly Property Domain() As String
        Get
            Return m_Domain
        End Get
    End Property

#Region "IDisposable Support"
    Private disposedValue As Boolean ' To detect redundant calls
    ' IDisposable
    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not Me.disposedValue Then
            If disposing Then
                ' TODO: dispose managed state (managed objects).
            End If
            Me.m_UserName = String.Empty
            Me.m_Password = String.Empty
            Me.m_Domain = String.Empty
        End If
        Me.disposedValue = True
    End Sub

    Protected Overrides Sub Finalize()
        Dispose(False)
        MyBase.Finalize()
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region

End Class