﻿Imports System.Text.RegularExpressions

Public Class frmMain

    Private WithEvents card As New K8055(enableEvents:=True, Interval:=100)
    ' Define control collections
    Private checkboxes(5) As CheckBox
    Private inputControls() As Control

    ' Timer properties
    Private Const TIMER_INTERVAL = 1    ' Set the timer interval (in seconds). You shouldn't change it.
    Private tickerCount As Long = 0

    ' Define the messages your message could give
    Private Const MSG_FTP_BUSY = "FTP Upload Failed (client busy)"
    Private Const MSG_FTP_UPLOADING = "Uploading webpage..."
    Private Const MSG_FTP_COMPLETED = "FTP Upload completed. Waiting for next cycle..."
    Private Const MSG_FTP_RUNNING = "AUTO Uploading Enabled. Waiting for first cycle..."
    Private Const MSG_FTP_INITIALIZE = "Program ready. Press 'run uploader' to start."
    Private Const MSG_FTP_UPLOADEDONCE = "FTP Upload completed. Press 'run uploader' for AUTO uploads."
    Private Const MSG_FTP_SINGLE_UPLOAD = "Uploading webpage..."
    Private Const MSG_FTP_SINGLE_UPLOAD_REFUSED = "Uploading webpage... Second request refused!"
    Private Const MSG_NO_K8055_FOUND = "The K8055 card was not found. Check connections."

    ' Define registry store location
    Private Const REGISTRY_PATH = "HKEY_CURRENT_USER\Software\Bvsystems\Web Uploader"

    ' Secret password key
    Private pwdKey As String

    ' Define the jumper settings for your card
    Private Const SK5 = True
    Private Const SK6 = True

    ' Credentials information
    Private _username As String
    Private _password As String
    Private savePassword As Boolean = True

    ' inputField types
    Private Enum inputType
        IP = 1
        HOSTNAME = 2
        REMOTE_FOLDER = 3
        REMOTE_FILE = 4
        USERNAME = 5
        PASSWORD = 6
    End Enum




#Region "!!! Defining the HTML variables !!!"
    ' In this sub you can define your own additional HTML variables. Each variable must be
    ' added to the htmlPage object.
    Public Sub defineVariables(ByRef htmlPage As WebPage)
        With htmlPage
            .addVariable("%INP1%", IIf(card.getDigitalInput(1), "ON", "OFF"))
            .addVariable("%INP2%", IIf(card.getDigitalInput(2), "ON", "OFF"))
            .addVariable("%INP3%", IIf(card.getDigitalInput(3), "ON", "OFF"))
            .addVariable("%INP4%", IIf(card.getDigitalInput(4), "ON", "OFF"))
            .addVariable("%INP5%", IIf(card.getDigitalInput(5), "ON", "OFF"))
            .addVariable("%DATE%", CStr(Now()))
        End With
    End Sub
#End Region




#Region "Program Initialisation"
    Private Sub frmMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Add all the checkboxes to a control Array
        checkboxes = New CheckBox() {INP1, INP2, INP3, INP4, INP5}

        ' Create a container that stores a reference to all the user controls.
        ' It is used to simplify further coding.
        inputControls = New Control() {inpHost, inpRemoteFolder, inpFileName, cmbInterval}


        ' Initialize Interface with the K8055
        card.connect(SK5, SK6)

        ' Add some intervals to the combolist
        With cmbInterval
            .Items.Add(New Interval("Each 10 seconds", 10))
            .Items.Add(New Interval("Each Minute", 60))
            .Items.Add(New Interval("Each 5 Minutes", 6 * 50))
            .Items.Add(New Interval("Each 10 Minutes", 10 * 60))
            .Items.Add(New Interval("Each 20 Minutes", 20 * 60))
            .Items.Add(New Interval("Each 30 Minutes", 30 * 60))
            .Items.Add(New Interval("Each 45 Minutes", 45 * 60))
            .Items.Add(New Interval("Each Hour", 60 * 60))
        End With

        ' Set default properties of the upload timer
        With UploadTimer
            .Enabled = True
            .Interval = TIMER_INTERVAL * 1000
        End With

        ' Initialize the ProgressBar
        IntelliBar_RunMarquee(False)
        status.Text = MSG_FTP_INITIALIZE

        ' Add an event handler that is fired each time one of the input controls is changed
        For Each ctrl As Control In inputControls
            AddHandler ctrl.TextChanged, AddressOf processChangedOptions
        Next

        ' Check if a connection between PC and Interface exists
        If (Not card.isConnected()) Then
            cmdRunUpdater.Enabled = False
            cmdUploadNow.Enabled = False
            status.Text = MSG_NO_K8055_FOUND
        End If

        ' read the settings from the Registry
        readSettingsFromRegistry()
        lblSave.Visible = False
    End Sub




    ' This event is called when one of the user options was changed.
    Private Sub processChangedOptions(ByVal sender As System.Object, ByVal e As System.EventArgs)
        lblSave.Visible = True
    End Sub
#End Region




#Region "K8055 Specific Functions"
    ' This method checks the Interface 
    Private Function checkInterface() As Boolean
        If Not card.isConnected Then
            If card.connect(SK5, SK6) Then
                cmdRunUpdater.Enabled = True
                cmdUploadNow.Enabled = True
                status.Text = MSG_FTP_INITIALIZE
                Return True
            Else
                status.Text = MSG_NO_K8055_FOUND
                Return False
            End If
        Else
            Return True
        End If
    End Function



    ' Change the inputs of the canal
    Private Sub card_digitalInputChanged(ByVal channel As Integer, ByVal Value As Boolean) Handles card.digitalInputChanged
        checkboxes(channel - 1).Checked = Value

        ' Check if the html page must be uploaded
        If chkUploadOnChange.Checked Then
            If UploadThread.IsBusy = False Then
                doUpload()
            Else
                status.Text = MSG_FTP_SINGLE_UPLOAD_REFUSED
            End If
        End If
    End Sub
#End Region




#Region "Periodical background uploading"
    ' This Event occurs each minute when the UploadTimer has ticked
    Private Sub UploadTimer_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles UploadTimer.Tick
        If checkInterface() = True Then
            If cmdRunUpdater.Checked Then
                tickerCount += TIMER_INTERVAL
                IntelliBar_updateValue()

                ' Retrieve the selected Interval from the comboBox
                Dim selectedInterval As Interval
                selectedInterval = cmbInterval.SelectedItem
                If Not (selectedInterval Is Nothing) Then
                    ' Something is selected...
                    If tickerCount > selectedInterval.interval Then
                        tickerCount = 0
                        ' Ok to Upload now!
                        doUpload()
                    End If
                End If
            End If
        End If
    End Sub



    ' This code will actual launch the uploading
    Private Sub doUpload()
        If Not UploadThread.IsBusy() Then
            ' Only continues if the uploader thread is not already working!
            ' First define an object to pass to the UploadThread procedure
            Dim parameters As New Parameters()
            parameters("username") = _username
            parameters("password") = _password
            parameters("host") = inpHost.Text
            parameters("remoteFolder") = inpRemoteFolder.Text
            parameters("webpage") = inpFileName.Text

            ' Declare the webpage object. Add it to a parameter afterwards.
            Dim htmlPage As New WebPage()
            defineVariables(htmlPage)

            ' Add it to the parameter object.
            parameters("htmlData") = htmlPage

            ' Start the upload
            status.Text = MSG_FTP_UPLOADING
            IntelliBar_RunMarquee(True)
            UploadThread.RunWorkerAsync(parameters)
        Else
            status.Text = MSG_FTP_BUSY
        End If
    End Sub



    ' The backgroundworker uploads your data in the background.
    Private Sub UploadThread_DoWork(ByVal sender As Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles UploadThread.DoWork

        If TypeOf e.Argument Is Parameters Then
            Dim v As Parameters
            v = CType(e.Argument, Parameters)
            Dim ftp As New FtpClient(v("host"), v("username"), v("password"))
            ftp.uploadFile(v("htmlData"), v("remoteFolder"), v("webpage"))
        Else
            Debug.Print("Uploaded failed: invalid object")
        End If
    End Sub


    ' Upload Completed!
    Private Sub UploadThread_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles UploadThread.RunWorkerCompleted
        status.Text = MSG_FTP_COMPLETED
        IntelliBar_RunMarquee(False)
    End Sub
#End Region




#Region "IntelliBar Coding"
    ' The Intellibar is the little bar at the bottom which reflects the state
    ' of the program. During an upload action, it will use a Marquee effect.
    ' When the program is just waiting for the next upload cycle, it will 
    ' indicate the duration.
    Private Sub IntelliBar_RunMarquee(ByVal enabled As Boolean)
        If enabled Then
            intelliBar.Style = ProgressBarStyle.Marquee
        Else
            intelliBar.Style = ProgressBarStyle.Blocks
        End If
    End Sub

    Private Sub IntelliBar_updateValue()
        If intelliBar.Style <> ProgressBarStyle.Marquee Then
            ' Only updates the intelliBar when the marquee effect is not working.

            ' How to calculate the time to the next synchronisation?
            ' progresbar.width must be set to the total interval between 2 synchronisations

            With intelliBar
                ' First we need to find out the selected interval. We get it from the
                ' combo box.
                Dim selectedInterval As Interval
                selectedInterval = cmbInterval.SelectedItem
                If Not (selectedInterval Is Nothing) Then
                    With intelliBar
                        .Minimum = 0
                        .Maximum = selectedInterval.interval    ' interval in seconds

                        If (selectedInterval.interval - tickerCount) > -1 Then
                            .Value = selectedInterval.interval - tickerCount
                        Else
                            .Value = 0
                        End If
                    End With
                End If
            End With
        End If
    End Sub
#End Region




#Region "Button Handling"
    ' Upload Now button
    Private Sub cmdUploadNow_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdUploadNow.Click
        If validateInputs() Then
            ' Only continue if all inputs were validated
            status.Text = MSG_FTP_SINGLE_UPLOAD
            doUpload()
        Else
            MsgBox("You have not entered all necessary parameters." & vbCrLf & _
                    "Complete them and try again.", _
                    MsgBoxStyle.OkOnly + MsgBoxStyle.Critical, _
                    "Incomplete setup!")
        End If
    End Sub



    ' Run Updater button (is in fact a checkbox that appears to be a Forms.Button object)
    Private Sub cmdRunUpdater_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdRunUpdater.CheckedChanged
        If cmdRunUpdater.Checked Then
            If validateInputs() Then
                ' Only continue if all inputs were validated
                UploadTimer.Enabled = True
                status.Text = MSG_FTP_RUNNING
            Else

                MsgBox("You have not entered all necessary parameters." & vbCrLf & _
                        "Complete them and try again.", _
                        MsgBoxStyle.OkOnly + MsgBoxStyle.Critical, _
                        "Incomplete setup!")

                cmdRunUpdater.Checked = False
            End If
        Else
            UploadTimer.Enabled = False
            If (Not UploadThread.IsBusy) Then IntelliBar_RunMarquee(False)
            intelliBar.Value = 0
            status.Text = MSG_FTP_INITIALIZE
        End If
    End Sub
#End Region




#Region "Input Fields Handling (Validating & Saving)"
    ' This function will validate the data from all textfields. It returns true when no errors occured.
    Private Function validateInputs() As Boolean
        Dim result As Boolean = True

        If validInputData(inpHost.Text, inputType.HOSTNAME) = False And validInputData(inpHost.Text, inputType.IP) = False Then
            ErrorProvider1.SetError(inpHost, "Please specify the host name of the FTP server." & vbCrLf & _
                                             "Example: ftp.microsoft.com" & vbCrLf & _
                                             "Example: 162.112.64.8")
            result = False
        Else
            ErrorProvider1.SetError(inpHost, "")
        End If


        If validInputData(inpRemoteFolder.Text, inputType.REMOTE_FOLDER) = False Then
            ErrorProvider1.SetError(inpRemoteFolder, "Please specify the name the remote folder where" & vbCrLf & _
                                                     "your file will be stored. This folder is relative" & vbCrLf & _
                                                     "to your FTP base folder! It should not contain an" & vbCrLf & _
                                                     "absolute path. There must always be a backslash at" & vbCrLf & _
                                                     "the end of the foldername. Type / to user the base folder." & vbCrLf & _
                                                     "Example: /public_html/" & vbCrLf & _
                                                     "Example: /")
            result = False
        Else
            ErrorProvider1.SetError(inpRemoteFolder, "")
        End If


        If validInputData(inpFileName.Text, inputType.REMOTE_FILE) = False Then
            ErrorProvider1.SetError(inpFileName, "Please specify the name of the html file" & vbCrLf & _
                                                 "which will be saved to the FTP server." & vbCrLf & _
                                                 "Example: k8055.html")
            result = False
        Else
            ErrorProvider1.SetError(inpFileName, "")
        End If


        If _username = "" Or _password = "" Then
            ErrorProvider1.SetError(inpUsername, "Please specify a valid login to the FTP server!" & vbCrLf & _
                                                 "click change credentials to set a login")
            result = False
        Else
            ErrorProvider1.SetError(inpUsername, "")
        End If


        If cmbInterval.SelectedItem Is Nothing Then
            ErrorProvider1.SetError(cmbInterval, "No valid interval was entered")
            result = False
        Else
            ErrorProvider1.SetError(cmbInterval, "")
        End If

        Return result
    End Function



    ' Called when the user checks the "upload on change" option 
    Private Sub chkUploadOnChange_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkUploadOnChange.CheckedChanged
        If chkUploadOnChange.Checked Then
            Dim ans As Integer

            ans = MsgBox("Warning! Use this function with care. It will execute a FTP upload " & vbCrLf & _
                         "each time a K8055 input changes. This may result in too many " & vbCrLf & _
                         "connections to your FTP server. The server might even block your IP" & vbCrLf & _
                         "address." & vbCrLf & _
                         "Are you sure you want to proceed?", _
                         MsgBoxStyle.Question + MsgBoxStyle.YesNoCancel, _
                         "Confirm activation of this option")

            If ans <> vbYes Then
                chkUploadOnChange.Checked = False
            Else
            End If
        End If
    End Sub



    Private Sub readSettingsFromRegistry()
        With My.Computer.Registry
            inpHost.Text = .GetValue(REGISTRY_PATH, "host", "")
            inpRemoteFolder.Text = .GetValue(REGISTRY_PATH, "remote folder", "")
            inpFileName.Text = .GetValue(REGISTRY_PATH, "filename", "")
            inpUsername.Text = .GetValue(REGISTRY_PATH, "username", "")
            _password = .GetValue(REGISTRY_PATH, "password", "")
            If _password <> "" Then
                Dim enc As New Encrypter("aSecredPassword")
                _password = enc.deCrypt(_password)
            End If

            _username = inpUsername.Text

            ' Find out the matching Interval object
            Dim selectedInterval As String
            selectedInterval = .GetValue(REGISTRY_PATH, "current interval", "")
            For i As Integer = 0 To cmbInterval.Items.Count() - 1
                If cmbInterval.Items(i).ToString = selectedInterval Then
                    cmbInterval.SelectedIndex = i
                End If
            Next
        End With
    End Sub



    Private Sub writeSettingsToRegistry()
        With My.Computer.Registry
            .SetValue(REGISTRY_PATH, "host", inpHost.Text)
            .SetValue(REGISTRY_PATH, "remote folder", inpRemoteFolder.Text)
            .SetValue(REGISTRY_PATH, "filename", inpFileName.Text)
            .SetValue(REGISTRY_PATH, "username", inpUsername.Text)

            If savePassword = True Then
                Dim enc As New Encrypter("aSecredPassword")
                .SetValue(REGISTRY_PATH, "password", enc.enCrypt(_password))
            Else
                .SetValue(REGISTRY_PATH, "password", "")
            End If

            If Not (cmbInterval.SelectedItem Is Nothing) Then
                .SetValue("HKEY_CURRENT_USER\Software\Bvsystems\Web Uploader", "current interval", cmbInterval.SelectedItem.ToString())
            End If

        End With
    End Sub



    Private Sub cmdSave_LinkClicked(ByVal sender As System.Object, ByVal e As System.Windows.Forms.LinkLabelLinkClickedEventArgs) Handles lblSave.LinkClicked
        Try
            writeSettingsToRegistry()
            lblSave.Visible = False
            MsgBox("Settings saved sucessfully!", MsgBoxStyle.OkOnly + MsgBoxStyle.Information, "Settings saved")
        Catch ex As Exception
            MsgBox("An error occured:" & vbCrLf & ex.StackTrace, MsgBoxStyle.OkOnly + MsgBoxStyle.Critical, "Error while saving")
        End Try
    End Sub



    Private Sub LinkLabel2_LinkClicked(ByVal sender As System.Object, ByVal e As System.Windows.Forms.LinkLabelLinkClickedEventArgs) Handles lblSetCredentials.LinkClicked
        Using prompt As New WindowsCredentialPrompt("K8055 Uploader", "Enter the credentials for your FTP account.", IntPtr.Zero)
            If prompt.Show = WindowsCredentialPrompt.PromptResult.OK Then
                _username = prompt.UserName
                _password = prompt.Password
                inpUsername.Text = _username

                Dim ans As Integer
                ans = MsgBox("Do you want to store the password on your computer?" & vbCrLf & _
                              "It may not be safe to do so.", _
                              MsgBoxStyle.YesNoCancel + MsgBoxStyle.DefaultButton3 + MsgBoxStyle.Question, _
                              "Store your password?")

                If ans <> vbYes Then
                    ' do not save the password
                    savePassword = False
                Else
                    ' save the password
                    savePassword = True
                End If


            Else
                Debug.Print("User Cancel")
            End If
        End Using
    End Sub



    ' This function will validate your input fields.
    Private Function validInputData(ByVal testString As String, ByVal type As inputType) As Boolean
        Dim r As New Regex("")

        Select Case type
            Case inputType.IP
                ' IP address must meet conditions
                r = New Regex("\b(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\b")

            Case inputType.HOSTNAME
                ' Hostname must meet conditions
                r = New Regex("^(([a-zA-Z]|[a-zA-Z][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z]|[A-Za-z][A-Za-z0-9\-]*[A-Za-z0-9])$")

            Case inputType.REMOTE_FOLDER
                ' Matches the remote folder
                r = New Regex("^(/[A-Za-z0-9_]+)*/$")

            Case inputType.REMOTE_FILE
                ' Filenames can contain letters, numbers, dashes and underscores. They must have an extension that contains letters or numbers.
                r = New Regex("^[a-zA-Z0-9\-_]*.[a-zA-Z0-9]*$")

            Case inputType.USERNAME
                ' Usernames can contain any character
                r = New Regex("")

            Case inputType.PASSWORD
                ' Passwords can contain any character
                r = New Regex("")
        End Select

        If Not (r Is Nothing) Then
            Return r.IsMatch(testString)
        Else
            Return False
        End If
    End Function
#End Region
End Class
